<?xml version="1.0" encoding="UTF-8"?>
<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
    xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance"
    xmlns:xs="http://www.w3.org/2001/XMLSchema" exclude-result-prefixes="xs"
    xmlns:xnl="urn:oasis:names:tc:ciq:xnl:4"
    xmlns:xal="urn:oasis:names:tc:ciq:xal:4"
    xmlns="urn:oasis:names:tc:evs:schema:eml" 
   version="2.0">
    
    <!-- Provided "as is" License: Creative Commons - http://creativecommons.org/licenses/by-sa/3.0/
        Author: David RR Webber, December, 2010 -->
    <!--
        Purpose  : Read in Google VIP project election XML and output OASIS EML v7.0 standard XML instances. 
        Original : 12/11/2010
        Initials : drrw
        CAM ver    : 1.1 
        Description : Part of a set of tools to provide open source tools for UOCAVA election management using OASIS EML.
        Updated  - Inits - Change
        12/11/10   drrw    Initial release
        05/29/11   drrw    Updates for latest EML v7.0 schema structures
        06/13/11   drrw    Adjust 505, 230 to latest schema
        
    -->
    <xsl:param name="ownerFileName">Virginia</xsl:param>
    <xsl:param name="electionYear">2010</xsl:param>
    <xsl:param name="electionName">Virginia Election 2010</xsl:param>
    <xsl:param name="schema">7.0</xsl:param>
    <xsl:param name="outputBoundaries">no</xsl:param>
    <xsl:param name="outputEML150">no</xsl:param>
    <xsl:param name="htmlReport">true</xsl:param>
    
    <xsl:output indent="yes"/>
    <xsl:output method="xml" indent="yes" omit-xml-declaration="yes"/>
    <xsl:output name="EMLfiles" method="xml" indent="yes" omit-xml-declaration="yes"/>
    
    <xsl:variable name="version">1.02</xsl:variable>
    <xsl:variable name="nbsp" ><xsl:text disable-output-escaping="yes">&#160;&#160;</xsl:text></xsl:variable><!-- this does not work - produces non-UTF-8 char -->
    <xsl:variable name="dateNow" select="substring(xs:string(current-date()),1,10)"/>
    <xsl:variable name="timeZone" select="substring(xs:string(current-date()),11,6)"/>
   
   <xsl:variable name="VIPfeed" select="/"/>
    
    <xsl:template match="/">
        <!-- Compute actual output path to write to --> 
       <xsl:message>Processing starting for VIP to EML converter</xsl:message>
        <xsl:variable name="writeResultPath110" select="concat($ownerFileName,'-EML110','.xml')"/>
        <xsl:variable name="eventIdentifier" select="distinct-values(vip_object/contest/election_id)"></xsl:variable>
        <xsl:message> - outputting EML 110</xsl:message>
        <xsl:message> -- total contests: <xsl:value-of select="count(vip_object/contest)"/></xsl:message>
        <!-- Output Election information -->
        <xsl:result-document exclude-result-prefixes="xs" format="EMLfiles" indent="yes"
            href="{$writeResultPath110}">
            
            <EML Id="110" SchemaVersion="{$schema}"
                xsi:schemaLocation="urn:oasis:names:tc:evs:schema:eml schema/110-electionevent-v7-0.xsd"
                >
                <xsl:comment>
                    Total Contests: <xsl:value-of select="count(vip_object/contest)"/>
                    Total Districts: <xsl:value-of select="count(distinct-values(vip_object/contest/electoral_district_id))"/>
                    Total Ballots: <xsl:value-of select="count(distinct-values(vip_object/contest/ballot_id))"/>
                     -----------------------------------------
                </xsl:comment>
                <EMLHeader>
                    <TransactionId>1</TransactionId>
                    <SequenceNumber>1</SequenceNumber>
                    <NumberInSequence>1</NumberInSequence>
                    <SequencedElementName>1</SequencedElementName>
                    <MessageLanguage>en-US</MessageLanguage>
                    <RequestedResponseLanguage>en-US</RequestedResponseLanguage>
                    <IssueDate><xsl:value-of select="current-date()"/></IssueDate>
                    <OfficialStatusDetail>
                        <OfficialStatus>approved</OfficialStatus>
                        <StatusDate><xsl:value-of select="current-date()"/></StatusDate>
                    </OfficialStatusDetail>
                </EMLHeader>
                <ElectionEvent>
                    <ElectionControl>
                        <EventIdentifier IdNumber="{$eventIdentifier}" DisplayOrder="1">
                            <EventName><xsl:value-of select="$electionYear"/> Election</EventName>
                            <EventQualifier IdNumber="{$electionYear}">General Election</EventQualifier>
                        </EventIdentifier>
                        <ElectionIdentifier ShortCode="{$electionYear}"
                            IdNumber="{$eventIdentifier}" DisplayOrder="1">
                            <ElectionName>
                                <xsl:value-of select="$electionName"/>
                            </ElectionName>
                            <ElectionCategory>General</ElectionCategory>
                        </ElectionIdentifier> 
                        <ElectionDate Type="Token">
                            <Start><xsl:value-of select="$dateNow"/>T06:00:00<xsl:value-of select="$timeZone"/></Start>
                            <End><xsl:value-of select="$dateNow"/>T19:00:00<xsl:value-of select="$timeZone"/></End>
                            <!-- SingleDate><xsl:value-of select="current-date()"/></SingleDate -->
                        </ElectionDate>
                        <ElectionScope>State Wide</ElectionScope>
                        <ElectionType>General</ElectionType>
                        <ElectionJurisdiction>State</ElectionJurisdiction>
                        <DocumentIdentifier Href="http://www.va.gov/SBE">Election Announcement</DocumentIdentifier>
                        <Languages>
                            <Language>en-US</Language>
                            <DefaultLanguage>en-US</DefaultLanguage>
                        </Languages>
                    </ElectionControl>
                    <ElectionAdministration>
                        <OfficeHours>9am to 5pm, Monday through Friday.</OfficeHours>
                        <OfficialSiteLinks>
                            <SiteLink Description="{$ownerFileName} State Board of Elections" DisplayOrder="1">http://www.va.gov/sbe</SiteLink>
                        </OfficialSiteLinks>
                        <ElectionOfficials>
                            <ElectionOfficial IdNumber="007">
                                <Responsibility>State Election Officer</Responsibility>
                                <PersonName id="1" LanguageCode="en-US">
                                    <xnl:PersonFullName>Mr Joe Legal</xnl:PersonFullName>
                                </PersonName>
                                <Contact DisplayOrder="1">
                                    <MailingAddress type="Business" addressID="001" usage="Business">
                                        <xal:FreeTextAddress>
                                            <xal:AddressLine type="City Hall" seqn="1">Election Offices, 100 Main Street, Richmond, VA</xal:AddressLine>
                                        </xal:FreeTextAddress>
                                        <!-- xal:GeoRSS/ -->
                                    </MailingAddress>
                                    <PreferredContact>Telephone</PreferredContact>
                                </Contact>
                            </ElectionOfficial>
                        </ElectionOfficials>
                    </ElectionAdministration>
                    <ElectionParticipation>
                        <Parties>
                            <xsl:variable name="PartiesFound" select="distinct-values(vip_object/candidate/party)"/>
                            <xsl:for-each select="$PartiesFound">
                                <Participant>
                                    <AffiliationIdentifier IdNumber="P{format-number(position(),'0000')}" DisplayOrder="{format-number(position(),'00000')}" ShortCode="{format-number(position(),'00000')}" ExpectedConfirmationReference="n/a">
                                        <RegisteredName><xsl:value-of select="."/></RegisteredName>
                                    </AffiliationIdentifier>
                                    <Type>Party</Type>
                                    <Description><xsl:value-of select="."/></Description>
                                </Participant>
                            </xsl:for-each>
                        </Parties>                               
                    </ElectionParticipation>
                    <Election>
                            <ElectionIdentifier ShortCode="{$electionYear}"
                                IdNumber="{$eventIdentifier}" DisplayOrder="1">
                                <ElectionName>
                                    <xsl:value-of select="$electionName"/>
                                </ElectionName>
                                <ElectionCategory>General</ElectionCategory>
                            </ElectionIdentifier> 
                          
                        <!-- Generate Contest Details -->
                        <xsl:apply-templates select="vip_object/contest"/>                    
                      
                    </Election>
                </ElectionEvent>
            </EML>
           
        </xsl:result-document>
        
        <xsl:variable name="writeResultPath230" select="concat($ownerFileName,'-EML230','.xml')"/>
        <!-- Output Candidate Information -->
        <xsl:message> - outputting EML 230</xsl:message>
        <xsl:message> -- total candidates: <xsl:value-of select="count(vip_object/candidate)"/></xsl:message>
        <xsl:result-document exclude-result-prefixes="xs" format="EMLfiles" indent="yes"
            href="{$writeResultPath230}">
            
            <EML Id="230" SchemaVersion="{$schema}"
                xsi:schemaLocation="urn:oasis:names:tc:evs:schema:eml schema/230-candidatelist-v7-0.xsd">
                <xsl:comment>
                    Total Candidates: <xsl:value-of select="count(vip_object/candidate)"/>
                    Total Candidate Id: <xsl:value-of select="count(distinct-values(vip_object/candidate/@id))"/>
                    Candidates Addresses: <xsl:value-of select="count(vip_object/candidate/filed_mailing_address[line1 &gt; ' '])"></xsl:value-of>
                    Total Districts: <xsl:value-of select="count(distinct-values(vip_object/contest/electoral_district_id))"/>
                    Total Ballots: <xsl:value-of select="count(distinct-values(vip_object/contest/ballot_id))"/>
                    -----------------------------------------
                </xsl:comment>
                <EMLHeader>
                    <TransactionId>01</TransactionId>
                    <SequenceNumber>01</SequenceNumber>
                    <NumberInSequence>01</NumberInSequence>
                    <SequencedElementName>WordToken</SequencedElementName>
                    <MessageLanguage>en-US</MessageLanguage>
                    <IssueDate><xsl:value-of select="current-date()"/></IssueDate>
                    <OfficialStatusDetail>
                        <OfficialStatus>approved</OfficialStatus>
                        <StatusDate><xsl:value-of select="current-date()"/></StatusDate>
                    </OfficialStatusDetail>
                </EMLHeader>
                <CandidateList>
                    <EventIdentifier IdNumber="{$eventIdentifier}" DisplayOrder="1">
                        <EventName><xsl:value-of select="$electionYear"/> Election</EventName>
                        <EventQualifier IdNumber="{$electionYear}">General Election</EventQualifier>
                    </EventIdentifier>
                    <EventDate>
                        <Start><xsl:value-of select="current-date()"/></Start>
                        <End><xsl:value-of select="current-date()"/></End>
                    </EventDate>
                    <ListDate><xsl:value-of select="current-date()"/></ListDate>
                   
                    <Election>
                        <ElectionIdentifier ShortCode="{$electionYear}"
                            IdNumber="{$eventIdentifier}" DisplayOrder="1">
                            <ElectionName>
                                <xsl:value-of select="$electionName"/>
                            </ElectionName>
                            <ElectionCategory>General</ElectionCategory>
                        </ElectionIdentifier> 
                        <Contest>
                            <ContestIdentifier ShortCode="ALL" IdNumber="0000" DisplayOrder="1">
                                <ContestName>ALL CONTESTS</ContestName>
                            </ContestIdentifier>
                            <Description>
                                <Message DisplayOrder="1" Lang="en-US">This is the consolidated list of all candidates for the election.</Message>
                            </Description>
                            <xsl:apply-templates select="vip_object/candidate" mode="EML230"/>
                        </Contest>
                        </Election>
                    </CandidateList>
            </EML>
        </xsl:result-document>
        
        <xsl:message> - outputting EML 505</xsl:message>
        
        <xsl:variable name="writeResultPath505" select="concat($ownerFileName,'-EML505','.xml')"/>
        <!-- Output Complete Election Information -->
        
        <xsl:result-document exclude-result-prefixes="xs" format="EMLfiles" indent="yes"
            href="{$writeResultPath505}">
          
            <EML Id="505" SchemaVersion="{$schema}"
                xsi:schemaLocation="urn:oasis:names:tc:evs:schema:eml schema/505-election-v7-0.xsd">
                <xsl:comment>
                    Total Localities: <xsl:value-of select="count(vip_object/locality)"/>
                    Total Districts: <xsl:value-of select="count(vip_object/electoral_district)"/>
                    Total Districts in Contests: <xsl:value-of select="count(distinct-values(vip_object/contest/electoral_district_id))"/>
                    Total Contests: <xsl:value-of select="count(vip_object/contest)"/>
                    Total Ballots: <xsl:value-of select="count(distinct-values(vip_object/ballot))"/>
                    Total Ballots in Contests: <xsl:value-of select="count(distinct-values(vip_object/contest/ballot_id))"/>
                    Total Parties: <xsl:value-of select="count(distinct-values(vip_object/candidate/party))"/>
                    Total Candidates: <xsl:value-of select="count(vip_object/candidate)"/>
                    Total Candidate Id: <xsl:value-of select="count(distinct-values(vip_object/candidate/@id))"/>
                    Candidates Addresses: <xsl:value-of select="count(vip_object/candidate/filed_mailing_address[line1 &gt; ' '])"></xsl:value-of>
                    Total Ballot measures: <xsl:value-of select="count(distinct-values(vip_object/contest/ballot_id))"/>
                    Total Polling Locations: <xsl:value-of select="count(distinct-values(vip_object/polling_location))"/>
                    Total Locality Boundaries: <xsl:value-of select="count(distinct-values(vip_object/locality_boundary))"/>
                    -----------------------------------------
                </xsl:comment>
  
                <EMLHeader>
                    <TransactionId>1</TransactionId>
                    <SequenceNumber>1</SequenceNumber>
                    <NumberInSequence>1</NumberInSequence>
                    <SequencedElementName>1</SequencedElementName>
                    <MessageLanguage>en-US</MessageLanguage>
                    <RequestedResponseLanguage>en-US</RequestedResponseLanguage>
                    <IssueDate><xsl:value-of select="current-date()"/></IssueDate>
                    <OfficialStatusDetail>
                        <OfficialStatus>approved</OfficialStatus>
                        <StatusDate><xsl:value-of select="current-date()"/></StatusDate>
                    </OfficialStatusDetail>
                </EMLHeader>
                <ElectionDetails>
                    <ElectionControl>
                        <EventIdentifier IdNumber="{$eventIdentifier}" DisplayOrder="1">
                            <EventName><xsl:value-of select="$electionYear"/> Election</EventName>
                            <EventQualifier IdNumber="{$electionYear}">General Election</EventQualifier>
                        </EventIdentifier>
                        <ElectionIdentifier ShortCode="{$electionYear}"
                            IdNumber="{$eventIdentifier}" DisplayOrder="1">
                            <ElectionName>
                                <xsl:value-of select="$electionName"/>
                            </ElectionName>
                            <ElectionCategory>General</ElectionCategory>
                        </ElectionIdentifier> 
                        <ElectionDate Type="Token">
                            <Start><xsl:value-of select="current-date()"/></Start>
                            <End><xsl:value-of select="current-date()"/></End>
                            <!-- SingleDate><xsl:value-of select="current-date()"/></SingleDate -->
                        </ElectionDate>
                        <ElectionScope>State Wide</ElectionScope>
                        <ElectionType>General</ElectionType>
                        <ElectionJurisdiction>State</ElectionJurisdiction>
                        <DocumentIdentifier Href="http://www.va.gov/SBE">Election Announcement</DocumentIdentifier>
                        <Languages>
                            <Language>en-US</Language>
                            <DefaultLanguage>en-US</DefaultLanguage>
                        </Languages>
                    </ElectionControl>
                    <ElectionAdministration>
                        <OfficeHours>9am to 5pm, Monday through Friday.</OfficeHours>
                        <OfficialSiteLinks>
                            <SiteLink Description="{$ownerFileName} State Board of Elections" DisplayOrder="1">http://www.va.gov/sbe</SiteLink>
                        </OfficialSiteLinks>
                        <ElectionOfficials>
                            <ElectionOfficial IdNumber="007">
                                <Responsibility>State Election Officer</Responsibility>
                                <PersonName id="1" LanguageCode="en-US">
                                    <xnl:PersonFullName>Mr Joe Legal</xnl:PersonFullName>
                                </PersonName>
                                <Contact DisplayOrder="1">
                                    <MailingAddress type="Business" addressID="001" usage="Business">
                                        <xal:FreeTextAddress>
                                            <xal:AddressLine type="City Hall" seqn="1">Election Offices, 100 Main Street, Richmond, VA</xal:AddressLine>
                                        </xal:FreeTextAddress>
                                        <!-- xal:GeoRSS/ -->
                                    </MailingAddress>
                                    <PreferredContact>Telephone</PreferredContact>
                                </Contact>
                            </ElectionOfficial>
                        </ElectionOfficials>
                    </ElectionAdministration>
                    <ElectionParticipation>
                        <Parties>
                            <xsl:variable name="PartiesFound" select="distinct-values(vip_object/candidate/party)"/>
                            <xsl:for-each select="$PartiesFound">
                                <Participant>
                                    <AffiliationIdentifier IdNumber="P{format-number(position(),'0000')}" DisplayOrder="{format-number(position(),'00000')}" ShortCode="{format-number(position(),'00000')}" ExpectedConfirmationReference="n/a">
                                        <RegisteredName><xsl:value-of select="."/></RegisteredName>
                                    </AffiliationIdentifier>
                                    <Type>Party</Type>
                                    <Description><xsl:value-of select="."/></Description>
                                </Participant>
                            </xsl:for-each>
                        </Parties>                               
                    </ElectionParticipation>
                    <Election id="{$electionYear}">
                        
                        <Localities>           
                            <!-- Output Locality information -->
                            <xsl:message> -- total locality: <xsl:value-of select="count(vip_object/locality)"/></xsl:message>                           
                            <xsl:apply-templates select="vip_object/locality" mode="EML505"/>            
                            <Precincts>
                                <xsl:message> -- total precincts: <xsl:value-of select="count(vip_object/precinct)"/></xsl:message>        
                                <xsl:apply-templates select="vip_object/precinct" mode="EML505"/> 
                            </Precincts>
                            <PrecinctSplits>
                                <xsl:message> -- total splits: <xsl:value-of select="count(vip_object/precinct_split)"/></xsl:message>        
                                <xsl:apply-templates select="vip_object/precinct_split" mode="EML505"/> 
                            </PrecinctSplits>
                        </Localities>
                        <Districts>
                            <xsl:message> -- total districts: <xsl:value-of select="count(vip_object/electoral_district)"/></xsl:message>        
                            <xsl:apply-templates select="vip_object/electoral_district" mode="EML505"/>                 
                        </Districts>
                        <Contests>
                            <xsl:message> -- total contests: <xsl:value-of select="count(vip_object/contest)"/></xsl:message>        
                            <xsl:apply-templates select="vip_object/contest" mode="EML505"/>   
                        </Contests>
                        <Ballots>
                            <xsl:message> -- total ballots: <xsl:value-of select="count(vip_object/ballot)"/></xsl:message>        
                            <xsl:apply-templates select="vip_object/ballot" mode="EML505a"/>
                        </Ballots>
                        <Candidates>
                            <xsl:message> -- total candidate: <xsl:value-of select="count(vip_object/candidate)"/></xsl:message>        
                            <xsl:apply-templates select="vip_object/candidate" mode="EML505"/>
                        </Candidates>
                        
                        <Propositions>
                            <Proposition ReferendumOptionIdentifier="Token" ProposalIdentifier="Token" Lang="en-US">
                                <ProposalText>string</ProposalText>
                                <SelectionText>string</SelectionText>
                                <StatementType>for</StatementType>
                                <PassThreshold>113876</PassThreshold>
                                <AbstainEffect>string</AbstainEffect>
                            </Proposition>
                        </Propositions>
                        
                        <PollingLocations>
                            <xsl:message> -- total polling locations: <xsl:value-of select="count(vip_object/polling_location)"/></xsl:message>        
                            <xsl:apply-templates select="vip_object/polling_location" mode="EML505"/>
                        </PollingLocations>
                        
                        <LocalityBoundaries>
                            <xsl:if test="$outputBoundaries = 'yes'">
                                <xsl:message> -- total street segments: <xsl:value-of select="count(vip_object/street_segment)"/></xsl:message>        
                            <xsl:apply-templates select="vip_object/street_segment" mode="EML505"/>
                            </xsl:if>
                        </LocalityBoundaries>
                    </Election>
                </ElectionDetails>
            </EML> 
        </xsl:result-document>
        
        <xsl:if test="$outputEML150 = 'yes'">
        <xsl:message> - outputting EML 150</xsl:message>
        <xsl:message> -- total street segments: <xsl:value-of select="count(vip_object/street_segment)"/></xsl:message>
        
        <xsl:variable name="writeResultPath150" select="concat($ownerFileName,'-EML150','.xml')"/>
        
        <!-- Output District Geoboundaries Information -->
        <xsl:result-document exclude-result-prefixes="xs" format="EMLfiles" indent="yes"
            href="{$writeResultPath150}">
            <EML Id="150" SchemaVersion="{$schema}"
                xsi:schemaLocation="urn:oasis:names:tc:evs:schema:eml schema/150-geodistrict-v7-0.xsd">
                <xsl:comment>
                    Total Precincts: <xsl:value-of select="count(vip_object/precinct)"/>
                    Total Precinct Splits: <xsl:value-of select="count(vip_object/precinct_split)"/>
                    Total Street Segments: <xsl:value-of select="count(vip_object/street_segment)"/>
                    -----------------------------------------
                </xsl:comment>
                <EMLHeader>
                    <TransactionId>01</TransactionId>
                    <SequenceNumber>01</SequenceNumber>
                    <NumberInSequence>01</NumberInSequence>
                    <SequencedElementName>WordToken</SequencedElementName>
                    <MessageLanguage>en-US</MessageLanguage>
                    <IssueDate><xsl:value-of select="current-date()"/></IssueDate>
                    <OfficialStatusDetail>
                        <OfficialStatus>approved</OfficialStatus>
                        <StatusDate><xsl:value-of select="current-date()"/></StatusDate>
                    </OfficialStatusDetail>
                </EMLHeader>
                <ElectionDistricts>
                    <ListDate><xsl:value-of select="current-date()"/></ListDate>
                    <Accepted>yes</Accepted>
                    <District>
                        <DistrictName><xsl:value-of select="$ownerFileName"/></DistrictName>
                        <OfficialStatusDetail>
                            <OfficialStatus>interim</OfficialStatus>
                            <StatusDate><xsl:value-of select="current-date()"/></StatusDate>
                        </OfficialStatusDetail>
                        <xsl:message> -- total precincts: <xsl:value-of select="count(vip_object/precinct)"/></xsl:message>        
                        <xsl:apply-templates select="vip_object/precinct" mode="EML150"/>
                        <xsl:message> -- total splits: <xsl:value-of select="count(vip_object/precinct_split)"/></xsl:message>  
                        <xsl:apply-templates select="vip_object/precinct_split" mode="EML150"/>
                        <xsl:message> -- total segments: <xsl:value-of select="count(vip_object/street_segment)"/></xsl:message>  
                        <xsl:apply-templates select="vip_object/street_segment" mode="EML150"/>
                    </District>
                </ElectionDistricts>
            </EML>
        </xsl:result-document>
        </xsl:if>
        <xsl:message> Processing completed</xsl:message>
    </xsl:template>
    
    <xsl:template match="contest">
        <Contest DisplayOrder="{format-number(position(),'00000')}">
            <ContestIdentifier ShortCode="{./@id}" IdNumber="{./@id}" DisplayOrder="{format-number(position(),'00000')}">
                <ContestName><xsl:value-of select="./office"/></ContestName>
            </ContestIdentifier>
            <!-- District>
                <ReportingUnitIdentifier IdNumber="{./electoral_district_id}" DisplayOrder="1"><xsl:value-of select="./electoral_district_id"/></ReportingUnitIdentifier>
                <Split>true</Split>
            </District -->
            <Area IdNumber="1" DisplayOrder="1">County</Area>
            <Position IdNumber="{./ballot_placement}" DisplayOrder="{./ballot_placement}"/>
            <VotingMethod>FPP</VotingMethod>
            <xsl:if test="./partisan='Yes'"><VotingMethod>Partisan</VotingMethod></xsl:if>
            <!-- ContestType>straight party</ContestType -->
            <MaxVotes>999</MaxVotes>
            <MaxWriteIn>999</MaxWriteIn>
            <NumberOfPositions>999</NumberOfPositions>
            <Messages>
                <Message Format="Text" Type="Voter" Lang="en-US">Please complete UOCAVA attestation statement.</Message>
            </Messages>
        </Contest>
        <!-- xsl:copy-of select="."/ -->
        <!-- contest id="500000">
            <election_id>14</election_id>
            <type>General</type>
            <electoral_district_id>501698</electoral_district_id>
            <partisan>No</partisan>
            <primary_party>&#x20;</primary_party>
            <office>Member School Board - Bayside</office>
            <ballot_id>500245</ballot_id>
            <ballot_placement>28</ballot_placement>
        </contest -->
    </xsl:template>
    
    <xsl:template match="candidate">
        <Affliation>
          <AffliationIdentifier ShortCode="{format-number(position(),'00000')}">
              <RegisteredName><xsl:value-of select="party"/></RegisteredName>
              <Type>Party</Type>
          </AffliationIdentifier>
          <Candidate DisplayOrder="{position()}" Independent="{if(party='Independent') then 'yes' else 'no'}">
              <CandidateIdentifier IdNumber="{@id}" DisplayOrder="1" ShortCode="{@id}" ExpectedConfirmationReference="n/a">
                <CandidateName><xsl:value-of select="name"/></CandidateName>
                <KnownAs/>
                <CandidateGender/>
                <ShortCode/>
            </CandidateIdentifier>
            <StatusDetails>
                <StatusItem setting="confirmed" date="{current-date()}"/>
            </StatusDetails>
            <DateOfBirth/>
            <Age/>
            <Gender/>
            <QualifyingAddress Type="Residence" ValidFrom="{current-date()}" LanguageCode="en-US">
                <xal:Locality Type="PostTown">
                    <xal:NameElement NameType="Name"><xsl:value-of select="filed_mailing_address/city"/></xal:NameElement>
                </xal:Locality>
                <xal:Thoroughfare>
                    <xal:NameElement><xsl:value-of select="filed_mailing_address/line1"/></xal:NameElement>
                    <xal:Number><xsl:value-of select="tokenize(filed_mailing_address/line1,' ')[1]"/></xal:Number>
                </xal:Thoroughfare>
                <xal:PostCode>
                    <xal:Identifier Type="ZIP"><xsl:value-of select="filed_mailing_address/zip"/></xal:Identifier>
                </xal:PostCode>
                <xal:AdminstrativeArea Type="State" ValidFrom="{current-date()}" LanguageCode="en-US">
                    <xal:NameElement><xsl:value-of select="filed_mailing_address/state"/></xal:NameElement>
                </xal:AdminstrativeArea>
            </QualifyingAddress>
            <Contact DisplayOrder="1">
                <Email Preferred="yes"><xsl:value-of select="email"/></Email>
                <Telephone/>
                <Fax/>               
            </Contact>
            <!-- Profession/ -->
        </Candidate>
       </Affliation> 
        
        <!-- xsl:copy-of select="."/ -->
        <!-- 
            <candidate id="500430">
            <name> Richard N. Early, Jr.</name>
            <party>Independent</party>
            <filed_mailing_address>
            <line1>237 Madison Rd</line1>
            <city>Stanardsville</city>
            <state>VA</state>
            <zip>229732933</zip>
            </filed_mailing_address>
            </candidate>        -->
    </xsl:template>
    
    <xsl:template match="candidate" mode="EML230">
            <Candidate DisplayOrder="{position()}" Independent="{if(party='Independent') then 'yes' else 'no'}">
                <CandidateIdentifier IdNumber="{@id}" DisplayOrder="1" ShortCode="{@id}" ExpectedConfirmationReference="n/a">
                    <CandidateName><xsl:value-of select="name"/></CandidateName>
                </CandidateIdentifier>
                <StatusDetails>
                    <StatusItem setting="candidate" date="{current-date()}"/>
                </StatusDetails>
                <QualifyingAddress type="Primary" status="current">
                    <xal:PostalAddress>
                        <xal:Thoroughfare><xsl:value-of select="filed_mailing_address/line1"/></xal:Thoroughfare>
                        <!-- xal:Number><xsl:value-of select="tokenize(filed_mailing_address/line1,' ')[1]"/></xal:Number -->
                        <xal:Locality type="PostTown"><xsl:value-of select="filed_mailing_address/city"/></xal:Locality>
                        <xal:AdministrativeArea type="State"><xsl:value-of select="filed_mailing_address/state"/></xal:AdministrativeArea>
                        <xal:PostCode type="ZIP"><xsl:value-of select="filed_mailing_address/zip"/></xal:PostCode>
                    </xal:PostalAddress>
                </QualifyingAddress>
                <Contact DisplayOrder="1">
                    <xsl:if test="string-length(email) &gt; 0"><Email Preferred="yes"><xsl:value-of select="email"/></Email></xsl:if>
                    <Telephone><Number>0</Number><Extension>0</Extension></Telephone>           
                </Contact>
                <!-- Profession/ -->
            </Candidate>
    </xsl:template>
    
    <xsl:template match="locality" mode="EML505">
        <LocalityEntity id="{./@id}" xsl:exclude-result-prefixes="xal xnl">
            <LocalityName><xsl:value-of select="./name"/></LocalityName>
            <LocalityType><xsl:value-of select="./type"/></LocalityType>
           <LocalityID>0-<xsl:value-of select="./@id"/></LocalityID>
        </LocalityEntity>
    </xsl:template>
   
   <xsl:template match="precinct" mode="EML505"><!-- this matches precinct details -->
      <Precinct id="{@id}"  xsl:exclude-result-prefixes="xal xnl">
         <PrecinctName><xsl:value-of select="./name"/></PrecinctName>
         <LocalityID><xsl:value-of select="./locality_id"/></LocalityID>
         <!-- ElectoralDistrictID/ -->
          <xsl:if test="./polling_location_id"><PollingLocationID><xsl:value-of select="./polling_location_id"/></PollingLocationID></xsl:if>
      </Precinct>
   </xsl:template>
   
   <xsl:template match="precinct_split" mode="EML505"><!-- this matches precinct details -->
    <PrecinctSplit id="{@id}"  xsl:exclude-result-prefixes="xal xnl">
       <SplitPrecinctName><xsl:value-of select="./name"/></SplitPrecinctName>
       <PrecinctID><xsl:value-of select="./precinct_id"/></PrecinctID>
       <xsl:apply-templates select="electoral_district_id" mode="EML505"/>
   </PrecinctSplit>
   </xsl:template>
   
   <xsl:template match="electoral_district_id" mode="EML505"><!-- this matches precinct details -->
   <ElectoralDistrictID  xsl:exclude-result-prefixes="xal xnl"><xsl:value-of select="."/></ElectoralDistrictID>
   </xsl:template>
   
    <xsl:template match="electoral_district" mode="EML505">
        <ElectoralDistrict id="{./@id}" xsl:exclude-result-prefixes="xal xnl">
            <DistrictName><xsl:value-of select="./name"/></DistrictName>
            <DistrictType><xsl:value-of select="./type"/></DistrictType>
        </ElectoralDistrict>
    </xsl:template>
   
   <xsl:template match="contest" mode="EML505">
      <Contest xsl:exclude-result-prefixes="xal xnl">
         <ContestIdentifier ShortCode="{./ballot_id}" IdNumber="{./@id}" DisplayOrder="{./ballot_placement}">
            <ContestName><xsl:value-of select="./office"/></ContestName>
         </ContestIdentifier>
         <ContestScope><xsl:value-of select="if(string-length(./primary_party) &gt; 3) then ./primary_party else 'State Wide'"/></ContestScope>
         <ElectionType><xsl:value-of select="./type"/></ElectionType>
         <ReportingUnitIdentifier IdNumber="{./electoral_district_id}" DisplayOrder="1">District</ReportingUnitIdentifier>
         <Qualified Reason="approved">yes</Qualified>
         <HowToVote>
             <Message DisplayOrder="1" Lang="en-US">Please see instructions online</Message>
         </HowToVote>
         <Rotation>no</Rotation>
         <xsl:if test="exists(./Rotation)"><RotationMethod>Rotation</RotationMethod></xsl:if>
         <VotingMethod>FPP</VotingMethod>
         <ChoiceType>Checkbox</ChoiceType>
         <MaxVotes>1</MaxVotes>
         <MinVotes>1</MinVotes>
         <MaxWriteIn>1</MaxWriteIn>
         <Blocked>"no"</Blocked>
         <BallotChoices Contested="yes" Partisan="{./partisan}">
            <xsl:variable name="ballotID" select="./ballot_id"/>
            <xsl:apply-templates select="$VIPfeed//vip_object/ballot[@id=$ballotID]" mode="EML505"/>
         </BallotChoices>
         <Messages>
             <Message DisplayOrder="1" Lang="en-US"/>
         </Messages>
         <ElectoralDistrictID><xsl:value-of select="./electoral_district_id"/></ElectoralDistrictID>
         <xsl:if test="string-length(./primary_party) &gt; 0 and(./primary_party &gt; ' ')"><PrimaryParty><xsl:value-of select="./primary_party"/></PrimaryParty></xsl:if>
         <ContestOffice><xsl:value-of select="./office"/></ContestOffice>
         <BallotID><xsl:value-of select="./ballot_id"/></BallotID>
         <BallotPlacement><xsl:value-of select="./ballot_placement"/></BallotPlacement>
      
     </Contest>
   </xsl:template>
   
   <xsl:template match="ballot" mode="EML505">
      <xsl:apply-templates select="candidate_id" mode="EML505"/>
   </xsl:template>
   
   <xsl:template match="candidate_id" mode="EML505"><!-- this matches candidates on contests and lookups up the candidate details -->
      <xsl:variable name="candidateLookup" select="."/>
      <xsl:variable name="candidateInfo" select="$VIPfeed//vip_object/candidate[@id=$candidateLookup]"/>
      <xsl:if test="count($candidateInfo/name) &gt; 1"><xsl:message>ERROR: duplicate candidate id <xsl:value-of select="$candidateLookup"/> for contest.</xsl:message></xsl:if>
      <xsl:if test="count($candidateInfo/name) &lt; 1"><xsl:message>ERROR: missing candidate id <xsl:value-of select="$candidateLookup"/> for contest.</xsl:message></xsl:if>
      <Candidate DisplayOrder="{position()}" Independent="{if($candidateInfo/candidate/party = 'Independent') then 'yes' else 'no'}"  xsl:exclude-result-prefixes="xal xnl">
          <CandidateIdentifier IdNumber="{.}" DisplayOrder="1" ShortCode="{.}" ExpectedConfirmationReference="n/a">
            <CandidateName><xsl:value-of select="$candidateInfo/name"/></CandidateName>
         </CandidateIdentifier>
          <StatusDetails>
              <StatusItem setting="candidate" date="{current-date()}"/>
          </StatusDetails>
      </Candidate>
   </xsl:template>
   
   <xsl:template match="candidate" mode="EML505"><!-- this matches candidate details -->
      <Candidate DisplayOrder="{position()}" Independent="{if(party='Independent') then 'yes' else 'no'}"  xsl:exclude-result-prefixes="xal xnl">
          <CandidateIdentifier IdNumber="{@id}" DisplayOrder="1" ShortCode="{@id}" ExpectedConfirmationReference="n/a">
            <CandidateName><xsl:value-of select="name"/></CandidateName>
         </CandidateIdentifier>
         <StatusDetails>
            <StatusItem setting="candidate" date="{current-date()}"/>
         </StatusDetails>
          <Contact DisplayOrder="{position()}" xsl:exclude-result-prefixes="xal xnl">
              <MailingAddress type="Primary" status="current" xsl:exclude-result-prefixes="xal xnl">
                  <xal:PostalAddress xsl:exclude-result-prefixes="xal xnl">
                      <xsl:choose>
                          <xsl:when test="tokenize(filed_mailing_address/line1,' ')[1]">
                              <xal:Thoroughfare number="{tokenize(filed_mailing_address/line1,' ')[1]}"><xsl:value-of select="filed_mailing_address/line1"/></xal:Thoroughfare>
                          </xsl:when>
                          <xsl:otherwise>
                              <xal:Thoroughfare><xsl:value-of select="filed_mailing_address/line1"/></xal:Thoroughfare>
                          </xsl:otherwise>
                      </xsl:choose>
                      
                      
                      <xal:Locality type="PostTown"><xsl:value-of select="filed_mailing_address/city"/></xal:Locality> 
                      <xal:AdministrativeArea type="State"><xsl:value-of select="filed_mailing_address/state"/></xal:AdministrativeArea> 
                      <xal:PostCode type="ZIP"><xsl:value-of select="filed_mailing_address/zip"/></xal:PostCode>
                      <xal:Country code="USA">United States</xal:Country> 
                  </xal:PostalAddress>
              </MailingAddress>
              <xsl:if test="string-length(email) &gt; 0"><Email><xsl:value-of select="email"/></Email></xsl:if>
              <Telephone><Number>0</Number><Extension>0</Extension></Telephone>           
          </Contact>
         <Party>
            <AffiliationIdentifier ExpectedConfirmationReference="n/a" ShortCode="PARTY" DisplayOrder="1" IdNumber="{format-number(position(),'000000')}">
               <RegisteredName><xsl:value-of select="party"/></RegisteredName>
            </AffiliationIdentifier>
            <Type>Party</Type>
            <!-- Description/ -->
         </Party>
      </Candidate>
   </xsl:template>
   
   <xsl:template match="polling_location" mode="EML505"><!-- This reports the ballot summary details only -->
     <PollingLocation IdNumber="{@id}" Channel="polling" DisplayOrder="{position()}"  xsl:exclude-result-prefixes="xal xnl">
      <PhysicalLocation IdNumber="{@id}">
         <PlaceName><xsl:value-of select="name"/></PlaceName>
          <Notes><Note>Polling times: <xsl:value-of select="normalize-space(substring-before(polling_hours,'-'))"/> - <xsl:value-of select="normalize-space(substring-after(polling_hours,'-'))"/></Note></Notes>
         <Address type="{if(contains(upper-case(name),'SCHOOL')) then 'EducationalInstitution' else 'Location'}">
            <xal:PostalAddress>
               <xal:Thoroughfare number="{tokenize(address/line1,' ')[1]}"><xsl:value-of select="address/line1"/></xal:Thoroughfare>
               <xal:Locality type="PostTown"><xsl:value-of select="address/city"/></xal:Locality>
               <xal:AdministrativeArea type="State"><xsl:value-of select="address/state"/></xal:AdministrativeArea>
               <xal:PostCode type="ZIP"><xsl:value-of select="address/zip"/></xal:PostCode>
               <xal:Country code="US">United States</xal:Country>
            </xal:PostalAddress>
            <!-- xal:GeoRSS/ -->
         </Address>
         <PollingStation/>
         <Map IdNumber="1" DisplayOrder="1"><URL>http://www.maps.fvap.mil</URL></Map> 
      </PhysicalLocation>
      <TimeAvailable pollingEventType="Absentee_Ballot">
          <Start><xsl:value-of select="$dateNow"/>T06:00:00<xsl:value-of select="$timeZone"/></Start>
          <End><xsl:value-of select="$dateNow"/>T19:00:00<xsl:value-of select="$timeZone"/></End>
      </TimeAvailable>
         <ContestAssociation IdNumber="{@id}" DisplayOrder="{position()}" />
         <ResultsReported CurrentStatus="none" ><Status/></ResultsReported>
         <PollAssociation/>
   </PollingLocation>
   </xsl:template>
   
   <xsl:template match="street_segment" mode="EML505"><!-- This reports the ballot summary details only -->
     <LocalityBoundary LocalityType="Street" PrecinctID="{precinct_id}" SplitID="{precinct_split_id}"  xsl:exclude-result-prefixes="xal xnl">
      <BoundaryMarker id="{@id}">
         <StartHouseNumber><xsl:value-of select="start_house_number"/></StartHouseNumber>
         <EndHouseNumber><xsl:value-of select="end_house_number"/></EndHouseNumber>
         <OddEvenBoth><xsl:value-of select="odd_even_both"/></OddEvenBoth>
         <NonHouseAddress  xsl:exclude-result-prefixes="xal xnl">
            <xal:PostalAddress>
               <xal:Thoroughfare type="{non_house_address/street_suffix}" name="{non_house_address/street_name}"><xsl:value-of select="concat(non_house_address/street_name,' ',non_house_address/street_suffix)"/></xal:Thoroughfare>
               <xal:Locality type="PostTown"><xsl:value-of select="non_house_address/city"/></xal:Locality>
               <xal:AdministrativeArea type="State"><xsl:value-of select="non_house_address/state"/></xal:AdministrativeArea>
               <xal:PostCode type="ZIP"><xsl:value-of select="non_house_address/zip"/></xal:PostCode>
               <xal:Country code="US">United States</xal:Country>
            </xal:PostalAddress>
            <!-- xal:GeoRSS/ -->
         </NonHouseAddress>
      </BoundaryMarker>
     </LocalityBoundary>
   </xsl:template>
   
   <xsl:template match="ballot" mode="EML505a"><!-- This reports the ballot summary details only -->
      <Ballot id="{./@id}"  xsl:exclude-result-prefixes="xal xnl">
        <xsl:apply-templates select="candidate_id" mode="EML505a"/>
         <!-- BallotMeasureID/ -->
      </Ballot>
   </xsl:template>
   
   <xsl:template match="candidate_id" mode="EML505a"><!-- this matches candidates on ballots and lookups up the candidate details -->
      <xsl:variable name="candidateLookup" select="."/>
      <xsl:variable name="candidateInfo" select="$VIPfeed//vip_object/candidate[@id=$candidateLookup]"/>
      <xsl:if test="count($candidateInfo/name) &gt; 1"><xsl:message>ERROR: duplicate candidate id <xsl:value-of select="$candidateLookup"/> for ballot.</xsl:message></xsl:if>
      <xsl:if test="count($candidateInfo/name) &lt; 1"><xsl:message>ERROR: missing candidate id <xsl:value-of select="$candidateLookup"/> for ballot.</xsl:message></xsl:if>
       <CandidateEntry>
           <CandidateIdentifier IdNumber="{.}" DisplayOrder="1" ShortCode="{.}" ExpectedConfirmationReference="n/a"  xsl:exclude-result-prefixes="xal xnl">
               <CandidateName><xsl:value-of select="$candidateInfo/name"/></CandidateName>
           </CandidateIdentifier>
       </CandidateEntry>
   </xsl:template>
    
    <xsl:template match="precinct" mode="EML150">
        <PollingDivision IdNumber="{@id}" >
            <Name divisionType="precinct"><xsl:value-of select="name"/></Name>
            <Association Id="{locality_id}"><xsl:value-of select="polling_location_id"/></Association>
        </PollingDivision>
    </xsl:template>
   
    <xsl:template match="precinct_split" mode="EML150">
        <PollingDivision IdNumber="{@id}" >
            <Name divisionType="precinct split"><xsl:value-of select="name"/></Name>
            <Association Id="{precinct_id}"><xsl:value-of select="@id"/></Association>
            <xsl:for-each select="electoral_district_id">
                <Association Id="{.}">Electoral district</Association>
            </xsl:for-each>
        </PollingDivision>
    </xsl:template>
    
    <xsl:template mode="EML150" match="street_segment">
        <xsl:variable name="findId" select="precinct_id"/>
        <xsl:variable name="loopStatus" select="xs:integer(position() div 500)"/>
        <xsl:if test="position() &gt; 99 and($loopStatus * 500 = position())"><xsl:message> --- <xsl:value-of select="position()"/> - <xsl:value-of select="precinct_id"/></xsl:message></xsl:if>
        <PollingDivisionBoundary IdNumber="{precinct_id}">
            <DivisionName><xsl:value-of select="$VIPfeed//vip_object/precinct[@id = $findId][1]/name"/></DivisionName><!-- Performance optimization - first match -->
            <BoundaryMarkers>
                <BoundaryMarker Mode="beginsAt" Id="{precinct_id}">
                    <StreetSegment NumberingMode="{if(odd_even_both='Both') then 'all' else lower-case(odd_even_both)}">
                        <StreetSegmentMember SegmentType="boundary" addressID="{@id}">
                            <xal:PostalAddress>
                                <xal:Thoroughfare number="{start_house_number}" name="{non_house_address/street_suffix}"><xsl:value-of select="non_house_address/street_name"/></xal:Thoroughfare>
                                <xal:Locality type="PostTown"><xsl:value-of select="non_house_address/city"/></xal:Locality>
                                <xal:AdministrativeArea type="State"><xsl:value-of select="non_house_address/state"/></xal:AdministrativeArea>
                                <xal:PostCode type="ZIP"><xsl:value-of select="non_house_address/zip"/></xal:PostCode>
                            </xal:PostalAddress>
                        </StreetSegmentMember>
                    </StreetSegment>
                </BoundaryMarker>
                <BoundaryMarker Mode="endsAt" Id="{precinct_id}">
                    <StreetSegment NumberingMode="{if(odd_even_both='Both') then 'all' else lower-case(odd_even_both)}">
                        <StreetSegmentMember SegmentType="boundary" addressID="{@id}">
                            <xal:PostalAddress>
                                <xal:Thoroughfare number="{end_house_number}" name="{non_house_address/street_suffix}"><xsl:value-of select="non_house_address/street_name"/></xal:Thoroughfare>
                                <xal:Locality type="PostTown"><xsl:value-of select="non_house_address/city"/></xal:Locality>
                                <xal:AdministrativeArea type="State"><xsl:value-of select="non_house_address/state"/></xal:AdministrativeArea>
                                <xal:PostCode type="ZIP"><xsl:value-of select="non_house_address/zip"/></xal:PostCode>
                            </xal:PostalAddress>
                        </StreetSegmentMember>
                    </StreetSegment>
                </BoundaryMarker>
            </BoundaryMarkers>
        </PollingDivisionBoundary>
    </xsl:template>
    
        <!-- street_segment id="557760">
            <start_house_number>1230</start_house_number>
            <end_house_number>1471</end_house_number>
            <odd_even_both>Both</odd_even_both>
            <precinct_id>503266</precinct_id>
            <precinct_split_id>507325</precinct_split_id>
            <non_house_address>
            <street_name>BOONE TRAIL</street_name>
            <street_suffix>RD</street_suffix>
            <city>Duffield</city>
            <state>VA</state>
            <zip>24244</zip>
            </non_house_address>
            </street_segment -->
    
</xsl:stylesheet>
